@@ -3,6 +3,8 @@
 
 """Core control stuff for coverage.py."""
 
+from __future__ import annotations
+
 import atexit
 import collections
 import contextlib
@@ -15,9 +17,15 @@ import threading
 import time
 import warnings
 
+from types import FrameType
+from typing import (
+    cast,
+    Any, Callable, Dict, IO, Iterable, Iterator, List, Optional, Tuple, Union,
+)
+
 from coverage import env
 from coverage.annotate import AnnotateReporter
-from coverage.collector import Collector, CTracer
+from coverage.collector import Collector, HAS_CTRACER
 from coverage.config import read_coverage_config
 from coverage.context import should_start_context_test_function, combine_context_switchers
 from coverage.data import CoverageData, combine_parallel_data
@@ -31,24 +39,24 @@ from coverage.jsonreport import JsonReporter
 from coverage.lcovreport import LcovReporter
 from coverage.misc import bool_or_none, join_regex, human_sorted
 from coverage.misc import DefaultValue, ensure_dir_for_file, isolate_module
+from coverage.multiproc import patch_multiprocessing
 from coverage.plugin import FileReporter
 from coverage.plugin_support import Plugins
 from coverage.python import PythonFileReporter
 from coverage.report import render_report
 from coverage.results import Analysis
 from coverage.summary import SummaryReporter
+from coverage.types import (
+    TConfigurable, TConfigSectionIn, TConfigValueIn, TConfigValueOut,
+    TFileDisposition, TLineNo, TMorf,
+)
 from coverage.xmlreport import XmlReporter
 
-try:
-    from coverage.multiproc import patch_multiprocessing
-except ImportError:                                         # pragma: only jython
-    # Jython has no multiprocessing module.
-    patch_multiprocessing = None
 
 os = isolate_module(os)
 
 @contextlib.contextmanager
-def override_config(cov, **kwargs):
+def override_config(cov: Coverage, **kwargs: TConfigValueIn) -> Iterator[None]:
     """Temporarily tweak the configuration of `cov`.
 
     The arguments are applied to `cov.config` with the `from_args` method.
@@ -66,7 +74,7 @@ def override_config(cov, **kwargs):
 DEFAULT_DATAFILE = DefaultValue("MISSING")
 _DEFAULT_DATAFILE = DEFAULT_DATAFILE  # Just in case, for backwards compatibility
 
-class Coverage:
+class Coverage(TConfigurable):
     """Programmatic access to coverage.py.
 
     To use::
@@ -88,10 +96,10 @@ class Coverage:
     """
 
     # The stack of started Coverage instances.
-    _instances = []
+    _instances: List[Coverage] = []
 
     @classmethod
-    def current(cls):
+    def current(cls) -> Optional[Coverage]:
         """Get the latest started `Coverage` instance, if any.
 
         Returns: a `Coverage` instance, or None.
@@ -104,25 +112,25 @@ class Coverage:
         else:
             return None
 
-    def __init__(
+    def __init__(                       # pylint: disable=too-many-arguments
         self,
-        data_file=DEFAULT_DATAFILE,
-        data_suffix=None,
-        cover_pylib=None,
-        auto_data=False,
-        timid=None,
-        branch=None,
-        config_file=True,
-        source=None,
-        source_pkgs=None,
-        omit=None,
-        include=None,
-        debug=None,
-        concurrency=None,
-        check_preimported=False,
-        context=None,
-        messages=False,
-    ):  # pylint: disable=too-many-arguments
+        data_file: Optional[Union[str, DefaultValue]] = DEFAULT_DATAFILE,
+        data_suffix: Optional[Union[str, bool]] = None,
+        cover_pylib: Optional[bool] = None,
+        auto_data: bool = False,
+        timid: Optional[bool] = None,
+        branch: Optional[bool] = None,
+        config_file: Union[str, bool] = True,
+        source: Optional[Iterable[str]] = None,
+        source_pkgs: Optional[Iterable[str]] = None,
+        omit: Optional[Union[str, Iterable[str]]] = None,
+        include: Optional[Union[str, Iterable[str]]] = None,
+        debug: Optional[Iterable[str]] = None,
+        concurrency: Optional[Union[str, Iterable[str]]] = None,
+        check_preimported: bool = False,
+        context: Optional[str] = None,
+        messages: bool = False,
+    ) -> None:
         """
         Many of these arguments duplicate and override values that can be
         provided in a configuration file.  Parameters that are missing here
@@ -214,13 +222,11 @@ class Coverage:
         # data_file=None means no disk file at all. data_file missing means
         # use the value from the config file.
         self._no_disk = data_file is None
-        if data_file is DEFAULT_DATAFILE:
+        if isinstance(data_file, DefaultValue):
             data_file = None
 
-        self.config = None
-
         # This is injectable by tests.
-        self._debug_file = None
+        self._debug_file: Optional[IO[str]] = None
 
         self._auto_load = self._auto_save = auto_data
         self._data_suffix_specified = data_suffix
@@ -229,21 +235,23 @@ class Coverage:
         self._warn_no_data = True
         self._warn_unimported_source = True
         self._warn_preimported_source = check_preimported
-        self._no_warn_slugs = None
+        self._no_warn_slugs: List[str] = []
         self._messages = messages
 
         # A record of all the warnings that have been issued.
-        self._warnings = []
+        self._warnings: List[str] = []
 
         # Other instance attributes, set later.
-        self._data = self._collector = None
-        self._plugins = None
-        self._inorout = None
+        self._debug: DebugControl
+        self._plugins: Plugins
+        self._inorout: InOrOut
+        self._data: CoverageData
+        self._collector: Collector
+        self._file_mapper: Callable[[str], str]
+
         self._data_suffix = self._run_suffix = None
-        self._exclude_re = None
-        self._debug = None
-        self._file_mapper = None
-        self._old_sigterm = None
+        self._exclude_re: Dict[str, str] = {}
+        self._old_sigterm: Optional[Callable[[int, Optional[FrameType]], Any]] = None
 
         # State machine variables:
         # Have we initialized everything?
@@ -282,7 +290,7 @@ class Coverage:
         if not env.METACOV:
             _prevent_sub_process_measurement()
 
-    def _init(self):
+    def _init(self) -> None:
         """Set all the initial state.
 
         This is called by the public methods to initialize state. This lets us
@@ -295,10 +303,8 @@ class Coverage:
 
         self._inited = True
 
-        # Create and configure the debugging controller. COVERAGE_DEBUG_FILE
-        # is an environment variable, the name of a file to append debug logs
-        # to.
-        self._debug = DebugControl(self.config.debug, self._debug_file)
+        # Create and configure the debugging controller.
+        self._debug = DebugControl(self.config.debug, self._debug_file, self.config.debug_file)
 
         if "multiprocessing" in (self.config.concurrency or ()):
             # Multi-processing uses parallel for the subprocesses, so also use
@@ -322,7 +328,7 @@ class Coverage:
             # this is a bit childish. :)
             plugin.configure([self, self.config][int(time.time()) % 2])
 
-    def _post_init(self):
+    def _post_init(self) -> None:
         """Stuff to do after everything is initialized."""
         if self._should_write_debug:
             self._should_write_debug = False
@@ -333,7 +339,7 @@ class Coverage:
         if self.config._crash and self.config._crash in short_stack(limit=4):
             raise Exception(f"Crashing because called by {self.config._crash}")
 
-    def _write_startup_debug(self):
+    def _write_startup_debug(self) -> None:
         """Write out debug info at startup if needed."""
         wrote_any = False
         with self._debug.without_callers():
@@ -357,7 +363,7 @@ class Coverage:
         if wrote_any:
             write_formatted_info(self._debug.write, "end", ())
 
-    def _should_trace(self, filename, frame):
+    def _should_trace(self, filename: str, frame: FrameType) -> TFileDisposition:
         """Decide whether to trace execution in `filename`.
 
         Calls `_should_trace_internal`, and returns the FileDisposition.
@@ -368,7 +374,7 @@ class Coverage:
             self._debug.write(disposition_debug_msg(disp))
         return disp
 
-    def _check_include_omit_etc(self, filename, frame):
+    def _check_include_omit_etc(self, filename: str, frame: FrameType) -> bool:
         """Check a file name against the include/omit/etc, rules, verbosely.
 
         Returns a boolean: True if the file should be traced, False if not.
@@ -384,7 +390,7 @@ class Coverage:
 
         return not reason
 
-    def _warn(self, msg, slug=None, once=False):
+    def _warn(self, msg: str, slug: Optional[str] = None, once: bool = False) -> None:
         """Use `msg` as a warning.
 
         For warning suppression, use `slug` as the shorthand.
@@ -393,31 +399,32 @@ class Coverage:
         slug.)
 
         """
-        if self._no_warn_slugs is None:
-            if self.config is not None:
+        if not self._no_warn_slugs:
+            # _warn() can be called before self.config is set in __init__...
+            if hasattr(self, "config"):
                 self._no_warn_slugs = list(self.config.disable_warnings)
 
-        if self._no_warn_slugs is not None:
-            if slug in self._no_warn_slugs:
-                # Don't issue the warning
-                return
+        if slug in self._no_warn_slugs:
+            # Don't issue the warning
+            return
 
         self._warnings.append(msg)
         if slug:
             msg = f"{msg} ({slug})"
-        if self._debug is not None and self._debug.should('pid'):
+        if hasattr(self, "_debug") and self._debug.should('pid'):
             msg = f"[{os.getpid()}] {msg}"
         warnings.warn(msg, category=CoverageWarning, stacklevel=2)
 
         if once:
+            assert slug is not None
             self._no_warn_slugs.append(slug)
 
-    def _message(self, msg):
+    def _message(self, msg: str) -> None:
         """Write a message to the user, if configured to do so."""
         if self._messages:
             print(msg)
 
-    def get_option(self, option_name):
+    def get_option(self, option_name: str) -> Optional[TConfigValueOut]:
         """Get an option from the configuration.
 
         `option_name` is a colon-separated string indicating the section and
@@ -428,14 +435,14 @@ class Coverage:
         selected.
 
         As a special case, an `option_name` of ``"paths"`` will return an
-        OrderedDict with the entire ``[paths]`` section value.
+        dictionary with the entire ``[paths]`` section value.
 
         .. versionadded:: 4.0
 
         """
         return self.config.get_option(option_name)
 
-    def set_option(self, option_name, value):
+    def set_option(self, option_name: str, value: Union[TConfigValueIn, TConfigSectionIn]) -> None:
         """Set an option in the configuration.
 
         `option_name` is a colon-separated string indicating the section and
@@ -460,17 +467,17 @@ class Coverage:
             branch = True
 
         As a special case, an `option_name` of ``"paths"`` will replace the
-        entire ``[paths]`` section.  The value should be an OrderedDict.
+        entire ``[paths]`` section.  The value should be a dictionary.
 
         .. versionadded:: 4.0
 
         """
         self.config.set_option(option_name, value)
 
-    def load(self):
+    def load(self) -> None:
         """Load previously-collected coverage data from the data file."""
         self._init()
-        if self._collector:
+        if hasattr(self, "_collector"):
             self._collector.reset()
         should_skip = self.config.parallel and not os.path.exists(self.config.data_file)
         if not should_skip:
@@ -479,15 +486,11 @@ class Coverage:
         if not should_skip:
             self._data.read()
 
-    def _init_for_start(self):
+    def _init_for_start(self) -> None:
         """Initialization for start()"""
         # Construct the collector.
-        concurrency = self.config.concurrency or []
+        concurrency: List[str] = self.config.concurrency or []
         if "multiprocessing" in concurrency:
-            if not patch_multiprocessing:
-                raise ConfigError(                      # pragma: only jython
-                    "multiprocessing is not supported on this Python"
-                )
             if self.config.config_file is None:
                 raise ConfigError("multiprocessing requires a configuration file")
             patch_multiprocessing(rcfile=self.config.config_file)
@@ -550,11 +553,11 @@ class Coverage:
 
         # Create the file classifying substructure.
         self._inorout = InOrOut(
+            config=self.config,
             warn=self._warn,
             debug=(self._debug if self._debug.should('trace') else None),
             include_namespace_packages=self.config.include_namespace_packages,
         )
-        self._inorout.configure(self.config)
         self._inorout.plugins = self._plugins
         self._inorout.disp_class = self._collector.file_disposition_class
 
@@ -569,11 +572,13 @@ class Coverage:
                 # The Python docs seem to imply that SIGTERM works uniformly even
                 # on Windows, but that's not my experience, and this agrees:
                 # https://stackoverflow.com/questions/35772001/x/35792192#35792192
-                self._old_sigterm = signal.signal(signal.SIGTERM, self._on_sigterm)
+                self._old_sigterm = signal.signal(      # type: ignore[assignment]
+                    signal.SIGTERM, self._on_sigterm,
+                )
 
-    def _init_data(self, suffix):
+    def _init_data(self, suffix: Optional[Union[str, bool]]) -> None:
         """Create a data file if we don't have one yet."""
-        if self._data is None:
+        if not hasattr(self, "_data"):
             # Create the data file.  We do this at construction time so that the
             # data file will be written into the directory where the process
             # started rather than wherever the process eventually chdir'd to.
@@ -586,7 +591,7 @@ class Coverage:
                 no_disk=self._no_disk,
             )
 
-    def start(self):
+    def start(self) -> None:
         """Start measuring code coverage.
 
         Coverage measurement only occurs in functions called after
@@ -618,7 +623,7 @@ class Coverage:
         self._started = True
         self._instances.append(self)
 
-    def stop(self):
+    def stop(self) -> None:
         """Stop measuring code coverage."""
         if self._instances:
             if self._instances[-1] is self:
@@ -627,7 +632,7 @@ class Coverage:
             self._collector.stop()
         self._started = False
 
-    def _atexit(self, event="atexit"):
+    def _atexit(self, event: str = "atexit") -> None:
         """Clean up on process shutdown."""
         if self._debug.should("process"):
             self._debug.write(f"{event}: pid: {os.getpid()}, instance: {self!r}")
@@ -636,7 +641,7 @@ class Coverage:
         if self._auto_save:
             self.save()
 
-    def _on_sigterm(self, signum_unused, frame_unused):
+    def _on_sigterm(self, signum_unused: int, frame_unused: Optional[FrameType]) -> None:
         """A handler for signal.SIGTERM."""
         self._atexit("sigterm")
         # Statements after here won't be seen by metacov because we just wrote
@@ -644,7 +649,7 @@ class Coverage:
         signal.signal(signal.SIGTERM, self._old_sigterm)    # pragma: not covered
         os.kill(os.getpid(), signal.SIGTERM)                # pragma: not covered
 
-    def erase(self):
+    def erase(self) -> None:
         """Erase previously collected coverage data.
 
         This removes the in-memory data collected in this session as well as
@@ -653,14 +658,14 @@ class Coverage:
         """
         self._init()
         self._post_init()
-        if self._collector:
+        if hasattr(self, "_collector"):
             self._collector.reset()
         self._init_data(suffix=None)
         self._data.erase(parallel=self.config.parallel)
-        self._data = None
+        del self._data
         self._inited_for_start = False
 
-    def switch_context(self, new_context):
+    def switch_context(self, new_context: str) -> None:
         """Switch to a new dynamic context.
 
         `new_context` is a string to use as the :ref:`dynamic context
@@ -681,13 +686,13 @@ class Coverage:
 
         self._collector.switch_context(new_context)
 
-    def clear_exclude(self, which='exclude'):
+    def clear_exclude(self, which: str = 'exclude') -> None:
         """Clear the exclude list."""
         self._init()
         setattr(self.config, which + "_list", [])
         self._exclude_regex_stale()
 
-    def exclude(self, regex, which='exclude'):
+    def exclude(self, regex: str, which: str = 'exclude') -> None:
         """Exclude source lines from execution consideration.
 
         A number of lists of regular expressions are maintained.  Each list
@@ -704,36 +709,37 @@ class Coverage:
         """
         self._init()
         excl_list = getattr(self.config, which + "_list")
+        assert isinstance(regex, str)
         excl_list.append(regex)
         self._exclude_regex_stale()
 
-    def _exclude_regex_stale(self):
+    def _exclude_regex_stale(self) -> None:
         """Drop all the compiled exclusion regexes, a list was modified."""
         self._exclude_re.clear()
 
-    def _exclude_regex(self, which):
-        """Return a compiled regex for the given exclusion list."""
+    def _exclude_regex(self, which: str) -> str:
+        """Return a regex string for the given exclusion list."""
         if which not in self._exclude_re:
             excl_list = getattr(self.config, which + "_list")
             self._exclude_re[which] = join_regex(excl_list)
         return self._exclude_re[which]
 
-    def get_exclude_list(self, which='exclude'):
-        """Return a list of excluded regex patterns.
+    def get_exclude_list(self, which: str = 'exclude') -> List[str]:
+        """Return a list of excluded regex strings.
 
         `which` indicates which list is desired.  See :meth:`exclude` for the
         lists that are available, and their meaning.
 
         """
         self._init()
-        return getattr(self.config, which + "_list")
+        return cast(List[str], getattr(self.config, which + "_list"))
 
-    def save(self):
+    def save(self) -> None:
         """Save the collected coverage data to the data file."""
         data = self.get_data()
         data.write()
 
-    def _make_aliases(self):
+    def _make_aliases(self) -> PathAliases:
         """Create a PathAliases from our configuration."""
         aliases = PathAliases(
             debugfn=(self._debug.write if self._debug.should("pathmap") else None),
@@ -745,7 +751,12 @@ class Coverage:
                 aliases.add(pattern, result)
         return aliases
 
-    def combine(self, data_paths=None, strict=False, keep=False):
+    def combine(
+        self,
+        data_paths: Optional[Iterable[str]] = None,
+        strict: bool = False,
+        keep: bool = False
+    ) -> None:
         """Combine together a number of similarly-named coverage data files.
 
         All coverage data files whose name starts with `data_file` (from the
@@ -785,7 +796,7 @@ class Coverage:
             message=self._message,
         )
 
-    def get_data(self):
+    def get_data(self) -> CoverageData:
         """Get the collected data.
 
         Also warn about various problems collecting data.
@@ -803,12 +814,12 @@ class Coverage:
             if not plugin._coverage_enabled:
                 self._collector.plugin_was_disabled(plugin)
 
-        if self._collector and self._collector.flush_data():
+        if hasattr(self, "_collector") and self._collector.flush_data():
             self._post_save_work()
 
         return self._data
 
-    def _post_save_work(self):
+    def _post_save_work(self) -> None:
         """After saving data, look for warnings, post-work, etc.
 
         Warn about things that should have happened but didn't.
@@ -835,12 +846,15 @@ class Coverage:
                 self._data.touch_files(paths, plugin_name)
 
     # Backward compatibility with version 1.
-    def analysis(self, morf):
+    def analysis(self, morf: TMorf) -> Tuple[str, List[TLineNo], List[TLineNo], str]:
         """Like `analysis2` but doesn't return excluded line numbers."""
         f, s, _, m, mf = self.analysis2(morf)
         return f, s, m, mf
 
-    def analysis2(self, morf):
+    def analysis2(
+        self,
+        morf: TMorf,
+    ) -> Tuple[str, List[TLineNo], List[TLineNo], List[TLineNo], str]:
         """Analyze a module.
 
         `morf` is a module or a file name.  It will be analyzed to determine
@@ -866,7 +880,7 @@ class Coverage:
             analysis.missing_formatted(),
         )
 
-    def _analyze(self, it):
+    def _analyze(self, it: Union[FileReporter, TMorf]) -> Analysis:
         """Analyze a single morf or code unit.
 
         Returns an `Analysis` object.
@@ -877,15 +891,17 @@ class Coverage:
         self._post_init()
 
         data = self.get_data()
-        if not isinstance(it, FileReporter):
-            it = self._get_file_reporter(it)
+        if isinstance(it, FileReporter):
+            fr = it
+        else:
+            fr = self._get_file_reporter(it)
 
-        return Analysis(data, self.config.precision, it, self._file_mapper)
+        return Analysis(data, self.config.precision, fr, self._file_mapper)
 
-    def _get_file_reporter(self, morf):
+    def _get_file_reporter(self, morf: TMorf) -> FileReporter:
         """Get a FileReporter for a module or file name."""
         plugin = None
-        file_reporter = "python"
+        file_reporter: Union[str, FileReporter] = "python"
 
         if isinstance(morf, str):
             mapped_morf = self._file_mapper(morf)
@@ -905,9 +921,10 @@ class Coverage:
         if file_reporter == "python":
             file_reporter = PythonFileReporter(morf, self)
 
+        assert isinstance(file_reporter, FileReporter)
         return file_reporter
 
-    def _get_file_reporters(self, morfs=None):
+    def _get_file_reporters(self, morfs: Optional[Iterable[TMorf]] = None) -> List[FileReporter]:
         """Get a list of FileReporters for a list of modules or file names.
 
         For each module or file name in `morfs`, find a FileReporter.  Return
@@ -923,12 +940,12 @@ class Coverage:
 
         # Be sure we have a collection.
         if not isinstance(morfs, (list, tuple, set)):
-            morfs = [morfs]
+            morfs = [morfs]     # type: ignore[list-item]
 
         file_reporters = [self._get_file_reporter(morf) for morf in morfs]
         return file_reporters
 
-    def _prepare_data_for_reporting(self):
+    def _prepare_data_for_reporting(self) -> None:
         """Re-map data before reporting, to get implicit 'combine' behavior."""
         if self.config.paths:
             mapped_data = CoverageData(warn=self._warn, debug=self._debug, no_disk=True)
@@ -937,19 +954,19 @@ class Coverage:
 
     def report(
         self,
-        morfs=None,
-        show_missing=None,
-        ignore_errors=None,
-        file=None,
-        omit=None,
-        include=None,
-        skip_covered=None,
-        contexts=None,
-        skip_empty=None,
-        precision=None,
-        sort=None,
-        output_format=None,
-    ):
+        morfs: Optional[Iterable[TMorf]] = None,
+        show_missing: Optional[bool] = None,
+        ignore_errors: Optional[bool] = None,
+        file: Optional[IO[str]] = None,
+        omit: Optional[Union[str, List[str]]] = None,
+        include: Optional[Union[str, List[str]]] = None,
+        skip_covered: Optional[bool] = None,
+        contexts: Optional[List[str]] = None,
+        skip_empty: Optional[bool] = None,
+        precision: Optional[int] = None,
+        sort: Optional[str] = None,
+        output_format: Optional[str] = None,
+    ) -> float:
         """Write a textual summary report to `file`.
 
         Each module in `morfs` is listed, with counts of statements, executed
@@ -974,7 +991,7 @@ class Coverage:
         If `skip_empty` is true, don't report on empty files (those that have
         no statements).
 
-        `contexts` is a list of regular expressions.  Only data from
+        `contexts` is a list of regular expression strings.  Only data from
         :ref:`dynamic contexts <dynamic_contexts>` that match one of those
         expressions (using :func:`re.search <python:re.search>`) will be
         included in the report.
@@ -1019,13 +1036,13 @@ class Coverage:
 
     def annotate(
         self,
-        morfs=None,
-        directory=None,
-        ignore_errors=None,
-        omit=None,
-        include=None,
-        contexts=None,
-    ):
+        morfs: Optional[Iterable[TMorf]] = None,
+        directory: Optional[str] = None,
+        ignore_errors: Optional[bool] = None,
+        omit: Optional[Union[str, List[str]]] = None,
+        include: Optional[Union[str, List[str]]] = None,
+        contexts: Optional[List[str]] = None,
+    ) -> None:
         """Annotate a list of modules.
 
         .. note::
@@ -1058,19 +1075,19 @@ class Coverage:
 
     def html_report(
         self,
-        morfs=None,
-        directory=None,
-        ignore_errors=None,
-        omit=None,
-        include=None,
-        extra_css=None,
-        title=None,
-        skip_covered=None,
-        show_contexts=None,
-        contexts=None,
-        skip_empty=None,
-        precision=None,
-    ):
+        morfs: Optional[Iterable[TMorf]] = None,
+        directory: Optional[str] = None,
+        ignore_errors: Optional[bool] = None,
+        omit: Optional[Union[str, List[str]]] = None,
+        include: Optional[Union[str, List[str]]] = None,
+        extra_css: Optional[str] = None,
+        title: Optional[str] = None,
+        skip_covered: Optional[bool] = None,
+        show_contexts: Optional[bool] = None,
+        contexts: Optional[List[str]] = None,
+        skip_empty: Optional[bool] = None,
+        precision: Optional[int] = None,
+    ) -> float:
         """Generate an HTML report.
 
         The HTML is written to `directory`.  The file "index.html" is the
@@ -1116,14 +1133,14 @@ class Coverage:
 
     def xml_report(
         self,
-        morfs=None,
-        outfile=None,
-        ignore_errors=None,
-        omit=None,
-        include=None,
-        contexts=None,
-        skip_empty=None,
-    ):
+        morfs: Optional[Iterable[TMorf]] = None,
+        outfile: Optional[str] = None,
+        ignore_errors: Optional[bool] = None,
+        omit: Optional[Union[str, List[str]]] = None,
+        include: Optional[Union[str, List[str]]] = None,
+        contexts: Optional[List[str]] = None,
+        skip_empty: Optional[bool] = None,
+    ) -> float:
         """Generate an XML report of coverage results.
 
         The report is compatible with Cobertura reports.
@@ -1150,20 +1167,22 @@ class Coverage:
 
     def json_report(
         self,
-        morfs=None,
-        outfile=None,
-        ignore_errors=None,
-        omit=None,
-        include=None,
-        contexts=None,
-        pretty_print=None,
-        show_contexts=None,
-    ):
+        morfs: Optional[Iterable[TMorf]] = None,
+        outfile: Optional[str] = None,
+        ignore_errors: Optional[bool] = None,
+        omit: Optional[Union[str, List[str]]] = None,
+        include: Optional[Union[str, List[str]]] = None,
+        contexts: Optional[List[str]] = None,
+        pretty_print: Optional[bool] = None,
+        show_contexts: Optional[bool] = None,
+    ) -> float:
         """Generate a JSON report of coverage results.
 
         Each module in `morfs` is included in the report.  `outfile` is the
         path to write the file to, "-" will write to stdout.
 
+        `pretty_print` is a boolean, whether to pretty-print the JSON output or not.
+
         See :meth:`report` for other arguments.
 
         Returns a float, the total percentage covered.
@@ -1186,13 +1205,13 @@ class Coverage:
 
     def lcov_report(
         self,
-        morfs=None,
-        outfile=None,
-        ignore_errors=None,
-        omit=None,
-        include=None,
-        contexts=None,
-    ):
+        morfs: Optional[Iterable[TMorf]] = None,
+        outfile: Optional[str] = None,
+        ignore_errors: Optional[bool] = None,
+        omit: Optional[Union[str, List[str]]] = None,
+        include: Optional[Union[str, List[str]]] = None,
+        contexts: Optional[List[str]] = None,
+    ) -> float:
         """Generate an LCOV report of coverage results.
 
         Each module in 'morfs' is included in the report. 'outfile' is the
@@ -1213,7 +1232,7 @@ class Coverage:
         ):
             return render_report(self.config.lcov_output, LcovReporter(self), morfs, self._message)
 
-    def sys_info(self):
+    def sys_info(self) -> Iterable[Tuple[str, Any]]:
         """Return a list of (key, value) pairs showing internal information."""
 
         import coverage as covmod
@@ -1221,7 +1240,7 @@ class Coverage:
         self._init()
         self._post_init()
 
-        def plugin_info(plugins):
+        def plugin_info(plugins: List[Any]) -> List[str]:
             """Make an entry for the sys_info from a list of plug-ins."""
             entries = []
             for plugin in plugins:
@@ -1234,8 +1253,8 @@ class Coverage:
         info = [
             ('coverage_version', covmod.__version__),
             ('coverage_module', covmod.__file__),
-            ('tracer', self._collector.tracer_name() if self._collector else "-none-"),
-            ('CTracer', 'available' if CTracer else "unavailable"),
+            ('tracer', self._collector.tracer_name() if hasattr(self, "_collector") else "-none-"),
+            ('CTracer', 'available' if HAS_CTRACER else "unavailable"),
             ('plugins.file_tracers', plugin_info(self._plugins.file_tracers)),
             ('plugins.configurers', plugin_info(self._plugins.configurers)),
             ('plugins.context_switchers', plugin_info(self._plugins.context_switchers)),
@@ -1245,7 +1264,7 @@ class Coverage:
             ('config_contents',
                 repr(self.config._config_contents) if self.config._config_contents else '-none-'
             ),
-            ('data_file', self._data.data_filename() if self._data is not None else "-none-"),
+            ('data_file', self._data.data_filename() if hasattr(self, "_data") else "-none-"),
             ('python', sys.version.replace('\n', '')),
             ('platform', platform.platform()),
             ('implementation', platform.python_implementation()),
@@ -1266,7 +1285,7 @@ class Coverage:
             ('command_line', " ".join(getattr(sys, 'argv', ['-none-']))),
         ]
 
-        if self._inorout:
+        if hasattr(self, "_inorout"):
             info.extend(self._inorout.sys_info())
 
         info.extend(CoverageData.sys_info())
@@ -1279,10 +1298,13 @@ class Coverage:
 if int(os.environ.get("COVERAGE_DEBUG_CALLS", 0)):              # pragma: debugging
     from coverage.debug import decorate_methods, show_calls
 
-    Coverage = decorate_methods(show_calls(show_args=True), butnot=['get_data'])(Coverage)
+    Coverage = decorate_methods(        # type: ignore[misc]
+        show_calls(show_args=True),
+        butnot=['get_data']
+    )(Coverage)
 
 
-def process_startup():
+def process_startup() -> Optional[Coverage]:
     """Call this at Python start-up to perhaps measure coverage.
 
     If the environment variable COVERAGE_PROCESS_START is defined, coverage
@@ -1325,7 +1347,7 @@ def process_startup():
         return None
 
     cov = Coverage(config_file=cps)
-    process_startup.coverage = cov
+    process_startup.coverage = cov      # type: ignore[attr-defined]
     cov._warn_no_data = False
     cov._warn_unimported_source = False
     cov._warn_preimported_source = False
@@ -1335,7 +1357,7 @@ def process_startup():
     return cov
 
 
-def _prevent_sub_process_measurement():
+def _prevent_sub_process_measurement() -> None:
     """Stop any subprocess auto-measurement from writing data."""
     auto_created_coverage = getattr(process_startup, "coverage", None)
     if auto_created_coverage is not None:
